{ Siro 29.10.2023
  Testen ob das Setzen von System.SetTextBuf eine Zeitersparnis bringt bei ReadLn
  ! nur für Windows wegen der Zeitmessungsfunktionen

  Die Zeile $define GetMemFreeMem
  einfach ausklammern, dann wird New und Dispose verwendet,
  ansonsten wird GetMem und FreeMem verwendet.
}

unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls,
  Windows; // wird benötigt für QueryPerformanceFrequency und QueryPerformanceCounter

type

  { TForm1 }

  TForm1 = class(TForm)
    Button1: TButton;
    OpenDialog1: TOpenDialog;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private

  public

  end;

var
  Form1: TForm1;

implementation

// folgende Zeile ausklammern, dann wird New und Dispose verwendet
// ansonsten GetMem und FreeMem
{$define GetMemFreeMem}


{$R *.lfm}

{ TForm1 }


const BUFFERSIZE = 60*1024;    // 60 KByte


type TBuf = packed Array[0..BUFFERSIZE-1] of Byte;  // der neue TextBuffer
var   Buf : ^TBuf;                                  // ein Zeiger auf den TextBuffer

var freq,tStart,tStop:Int64;     // Variablen für die Zeitmessung

procedure TForm1.FormCreate(Sender: TObject);
begin
  ReturnNilIfGrowHeapFails:=TRUE;   // Damit KEIN Runtime Error ausgelöst wird bei GetMem
                                    // GetMem liefert dann bei Problemen ein NIL
  QueryPerformanceFrequency(freq);  // liefert die Frequent der Windows Performance Counters
end;

procedure TForm1.Button1Click(Sender: TObject);
var f:TextFile; s:string;
begin
  if NOT OpenDialog1.execute
    then exit;                            // Dateiauswahl wurde abgebrochen

  Buf:=nil;
  {$IFDEF GetMemFreeMem}
    Buf:=GetMem(BufferSize);              // Speicher fuer den Textpuffer reservieren mittels GetMem
    if Buf = NIL then begin               // wenns nicht geklappt hat dann
      Caption:='Fehler bei GetMem';       // Fehler ausgeben in der Titelleiste
      exit;                               // und Ende
    end;
  {$ELSE}
    New(Buf);                             // Speicher fuer den Textpuffer reservieren mittels New
  {$ENDIF}

  System.SetTextBuf(f,Buf^,SizeOf(TBuf));  // Dem System den Speicher zuweisen

  AssignFile(f,OpenDialog1.FileName);
  Reset(F);

  QueryPerformanceCounter(tStart);       // Startzeit festhalten
  While not eof(f) do begin              // die komplette Datei
    ReadLn(f,s);                         // Zeilenweise einlesen
  end;
  QueryPerformanceCounter(tStop);        // Endzeit festhalten

  Caption:=FloatToStr((tStop-tStart) / freq)+' Sekunden';  // Zeit in Sekunden ausgeben

  CloseFile(f);                      // Datei wieder schiessen

  {$IFDEF GetMemFreeMem}
    FreeMem(Buf,BufferSize);         // Speicher wieder freigeben mittels FreeMem
  {$ELSE}
    Dispose(Buf);                    // Speicher wieder freigeben mittels Dispose
  {$ENDIF}
end;

end.

